/*
 * osdlist.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <KDebug>

#include "osdlist.h"



OsdList::OsdList(  OsdConfig *conf ) : OsdObject( conf )
{
	scroll = 0;

	bar = new OsdVScrollBar( conf );
	connect( bar, SIGNAL(scrolled(int)), this, SLOT(barScrolled(int)) );
}



void OsdList::appendEntry( OsdListItem *it )
{
	itemList.append( it );
	connect( it, SIGNAL(wantRepaint()), this, SIGNAL(wantRepaint()) );
}



void OsdList::clear()
{
	int i;
	for ( i=0; i<itemList.count(); ++i )
		delete itemList.at(i);
	itemList.clear();
	scroll = 0;
}



OsdList::~OsdList()
{
	clear();
	delete bar;
}



void OsdList::mouseMove( QPoint &p )
{
	if ( !inMe( p.x(), p.y() ) && !hasFocus )
		return;
	else if ( !inMe( p.x(), p.y() ) )
		hasFocus = false;
	else
		hasFocus = true;

	if ( bar->isScrolling() ) {
		bar->mouseMove( p );
		return;
	}

	if ( !itemList.count() )
		return;

	QPoint qp( p.x(), p.y()+scroll );
	int itemH = itemList.at(0)->height()+MARGINSIZE;
	int i = scroll/itemH;
	if ( i<0 ) i =0;
	QRect clip( rect.x()+MARGINSIZE, rect.y()+MARGINSIZE, rect.width()-(2*MARGINSIZE), rect.height()-(2*MARGINSIZE) );
	for ( ; i<itemList.size(); ++i ) {
		if ( itemList.at(i)->y()>(scroll+clip.y()+clip.height()) )
			break;
		itemList.at(i)->mouseMove( qp );
	}
}



void OsdList::wheelEvent( QWheelEvent *e )
{
	if ( !hasFocus )
		return;
	int off = -e->delta()/8;
	if ( off<0 && scroll==0 )
		return;
	scroll+=off;
	if ( scroll<0 )
		scroll = 0;
	wantRepaint();
}



bool OsdList::mouseClicked( QMouseEvent *e )
{
	int i;

	if ( !inMe( e->x(), e->y() ) )
		return false;
	if ( bar->mouseClicked( e ) )
		return true;
	QMouseEvent me(e->type(),QPoint(e->x(),e->y()+scroll),e->button(),e->buttons(),e->modifiers());
	for ( i=0; i<itemList.size(); ++i ) {
		if ( itemList.at(i)->mouseClicked( &me ) )
			return true;
	}

	return false;
}



void OsdList::mouseReleased( QMouseEvent *e )
{
	bar->mouseReleased( e );
}



void OsdList::barScrolled( int off )
{
	scroll = off;
	emit wantRepaint();
}



void OsdList::paint()
{
	if ( rect.height()<=0 )
		return;

	if ( !itemList.count() )
		return;

	QRect clip( rect.x()+MARGINSIZE, rect.y()+MARGINSIZE, rect.width()-(2*MARGINSIZE)-bar->width(), rect.height()-(2*MARGINSIZE) );
	int itemH = itemList.at(0)->height()+MARGINSIZE;
	if ( (itemH*itemList.size())>clip.height() ) {
		int scrollmax = itemH*itemList.size()-clip.height();
		if ( scroll>scrollmax )
			scroll = scrollmax;
	}
	else
		scroll = 0;

	bar->setRange( itemH*itemList.size() );
	bar->setPos( scroll );
	bar->paint();

	glScissor( clip.x(), clip.y(), clip.width(), clip.height() );
	glEnable( GL_SCISSOR_TEST );
	glPushMatrix();
	glTranslatef( 0, -scroll, 0 );
	int i = scroll/itemH;
	if ( i<0 ) i =0;
	for ( ; i<itemList.size(); ++i ) {
		if ( itemList.at(i)->y()>(scroll+clip.y()+clip.height()) )
			break;
		itemList.at(i)->paint();
	}
	glPopMatrix();
	glDisable( GL_SCISSOR_TEST );
}



void OsdList::resize( int w, int h )
{
	rect.setHeight( h );
	rect.setWidth( w );
	QFontMetrics fm( config->normalFont() );
	QRect r = fm.boundingRect( QRect(), Qt::AlignVCenter, "Pepe" );
	//int listlen = (r.height()+MARGINSIZE)*itemList.size();
	//if ( listlen>(rect.height()-(2*MARGINSIZE)) )
		bar->setGeometry( rect.x()+rect.width()-MARGINSIZE-SCROLLBARSIZE, rect.y()+MARGINSIZE, SCROLLBARSIZE, rect.height()-(2*MARGINSIZE) );
	//else
	//	bar->setGeometry( QRect() );
	for ( int i=0; i<itemList.size(); ++i ) {
		itemList.at(i)->setGeometry( rect.x()+MARGINSIZE, rect.y()+MARGINSIZE+i*(r.height()+MARGINSIZE), rect.width()-(2*MARGINSIZE)-bar->width(), r.height() );
	}
}
